@@ -38,17 +38,17 @@ class ADOdbLoadBalancer
     /**
      * @var bool|array    All connections to each database.
      */
-    protected $connections = false;
+    protected $connections = [];
 
     /**
      * @var bool|array    Just connections to the write capable database.
      */
-    protected $connections_write = false;
+    protected $connections_write = [];
 
     /**
      * @var bool|array    Just connections to the readonly database.
      */
-    protected $connections_readonly = false;
+    protected $connections_readonly = [];
 
     /**
      * @var array    Counts of all connections and their types.
@@ -73,12 +73,12 @@ class ADOdbLoadBalancer
     /**
      * @var bool    Session variables that must be maintained across all connections, ie: SET TIME ZONE.
      */
-    protected $session_variables = false;
+    protected $session_variables = [];
 
     /**
      * @var bool    Called immediately after connecting to any DB.
      */
-    protected $user_defined_session_init_sql = false;
+    protected $user_defined_session_init_sql = [];
 
 
     /**
@@ -174,7 +174,7 @@ class ADOdbLoadBalancer
      * @param  string $type Type of database connection, either: 'write' capable or 'readonly'
      * @return bool|int|string
      */
-    private function getConnectionByWeight($type)
+    public function getConnectionByWeight($type)
     {
         if ($type == 'readonly') {
             $total_weight = $this->total_connection_weights['all'];
@@ -233,7 +233,7 @@ class ADOdbLoadBalancer
      * @return bool|ADOConnection
      * @throws Exception
      */
-    private function _getConnection($connection_id)
+    public function getConnectionById($connection_id)
     {
         if (isset($this->connections[$connection_id])) {
             $connection_obj = $this->connections[$connection_id];
@@ -261,6 +261,15 @@ class ADOdbLoadBalancer
                     throw $e; // No connections left, reThrow exception so application can catch it.
                 }
 
+                // Check to see if a connection test callback was defined, and if so execute it.
+                // This is useful for testing replication lag and such to ensure the connection is suitable to be used.
+                $test_connection_callback = $connection_obj->getConnectionTestCallback();
+                if (is_callable($test_connection_callback)
+                    && $test_connection_callback($connection_obj, $adodb_obj) !== TRUE
+                ) {
+                    return false;
+                }
+
                 if (is_array($this->user_defined_session_init_sql)) {
                     foreach ($this->user_defined_session_init_sql as $session_init_sql) {
                         $adodb_obj->Execute($session_init_sql);
@@ -298,9 +307,12 @@ class ADOdbLoadBalancer
 
             if ($connection_id !== false) {
                 try {
-                    $adodb_obj = $this->_getConnection($connection_id);
-                    // $connection_obj = $this->connections[$connection_id];
-                    break;
+                    $adodb_obj = $this->getConnectionById($connection_id);
+                    if (is_object($adodb_obj)) {
+                        break; //Found valid connection, continue with it.
+                    } else {
+                        throw new Exception('ADODB Connection Object does not exist. Perhaps LoadBalancer Database Connection Test Failed?');
+                    }
                 } catch (Exception $e) {
                     // Connection error, see if there are other connections to try still.
                     $this->removeConnection($connection_id);
@@ -315,6 +327,10 @@ class ADOdbLoadBalancer
             }
         }
 
+        if (!isset($connection_id)) {
+            throw new Exception('No connection available to use at this time! Type: ' . $type);
+        }
+
         $this->last_connection_id[$type] = $connection_id;
 
         if ($pin_connection === true) {
@@ -387,7 +403,7 @@ class ADOdbLoadBalancer
      */
     private function executeSessionVariables($adodb_obj = false)
     {
-        if (is_array($this->session_variables)) {
+        if (is_array($this->session_variables) && count($this->session_variables) > 0) {
             $sql = '';
             foreach ($this->session_variables as $name => $value) {
                 // $sql .= 'SET SESSION '. $name .' '. $value;
@@ -432,7 +448,7 @@ class ADOdbLoadBalancer
                         && $connection_obj->getADOdbObject()->_connectionID !== false
                     )
                 ) {
-                    $adodb_obj = $this->_getConnection($key);
+                    $adodb_obj = $this->getConnectionById($key);
                     if (is_object($adodb_obj)) {
                         $result_arr[] = $adodb_obj->Execute($sql, $inputarr);
                     }
@@ -595,6 +611,7 @@ class ADOdbLoadBalancer
             case 'binddate':
             case 'bindtimestamp':
             case 'setfetchmode':
+            case 'setcustommetatype':
                   $type = false; // No connection necessary.
                 break;
 
@@ -686,6 +703,11 @@ class ADOdbLoadBalancerConnection
      */
     protected $adodb_obj = false;
 
+    /**
+     * @var callable    Closure
+     */
+    protected $connection_test_callback = NULL;
+
     /**
      * @var string    Type of connection, either 'write' capable or 'readonly'
      */
@@ -761,6 +783,24 @@ class ADOdbLoadBalancerConnection
         return true;
     }
 
+    /**
+     * Anonymous function that is called and must return TRUE for the connection to be usable.*
+     *   The first argument is the type of connection to test.
+     *   Useful to check things like replication lag.
+     * @param callable $callback
+     * @return void
+     */
+    function setConnectionTestCallback($callback) {
+        $this->connection_test_callback = $callback;
+    }
+
+    /**
+     * @return callable|null
+     */
+    function getConnectionTestCallback() {
+        return $this->connection_test_callback;
+    }
+
     /**
      * Returns the ADODB object for this connection.
      *
