@@ -1,17 +1,23 @@
 <?php
-
-
-/*
-@version   v5.20.19  13-Dec-2020
-@copyright (c) 2000-2013 John Lim (jlim#natsoft.com). All rights reserved.
-@copyright (c) 2014      Damien Regad, Mark Newnham and the ADOdb community
-         Contributed by Ross Smith (adodb@netebb.com).
-  Released under both BSD license and Lesser GPL library license.
-  Whenever there is any discrepancy between the two licenses,
-  the BSD license will take precedence.
-	  Set tabs to 4 for best viewing.
-
-*/
+/**
+ * ADOdb Session Management
+ *
+ * This file is part of ADOdb, a Database Abstraction Layer library for PHP.
+ *
+ * @package ADOdb
+ * @link https://adodb.org Project's web site and documentation
+ * @link https://github.com/ADOdb/ADOdb Source code and issue tracker
+ *
+ * The ADOdb Library is dual-licensed, released under both the BSD 3-Clause
+ * and the GNU Lesser General Public Licence (LGPL) v2.1 or, at your option,
+ * any later version. This means you can use it in proprietary products.
+ * See the LICENSE.md file distributed with this source code for details.
+ * @license BSD-3-Clause
+ * @license LGPL-2.1-or-later
+ *
+ * @copyright 2000-2013 John Lim
+ * @copyright 2014 Damien Regad, Mark Newnham and the ADOdb community
+ */
 
 if (!function_exists('gzcompress')) {
 	trigger_error('gzip functions are not available', E_USER_ERROR);
@@ -38,8 +44,8 @@ class ADODB_Compress_Gzip {
 	/**
 	 */
 	function setLevel($level) {
-		assert('$level >= 0');
-		assert('$level <= 9');
+		assert($level >= 0);
+		assert($level <= 9);
 		$this->_level = (int) $level;
 	}
 
@@ -52,7 +58,7 @@ class ADODB_Compress_Gzip {
 	/**
 	 */
 	function setMinLength($min_length) {
-		assert('$min_length >= 0');
+		assert($min_length >= 0);
 		$this->_min_length = (int) $min_length;
 	}
 
